﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Operadores_LINQ
{
    class Program
    {
        static void Main(string[] args)
        {
            //Agregacao();
            //Concatenacao();
            //Conversao();
            //OperacaoElemento();
            //Igualdade();
            //Particionamento();
            //Projecao();
            Filtragem();
            Console.ReadKey();
        }

        public static void Filtragem()
        {
            //Sorting
            string[] nome = {"um","dois","cinco" };

            //Ordena a coleção alfabeticamente - primeiro caractere
            nome.OrderBy(f => f).ToArray();  //one three two

            // arranja alfabeticamente pelo segundo caractere
            nome.OrderBy(f => f[1]); // three one two

            //Ordena pelo tamanho
            nome.OrderBy(f => f.Length);  //one two three

            //Ordena pelo tamanho descendente
            nome.OrderByDescending(f => f.Length);

            // Primeiro pelo tamanho depois Descendente
            nome.OrderBy(f => f.Length).ThenByDescending(f => f); //two one three
        }

        public static void Projecao()
        {
            string[] nome = { "um", "dois", "cinco" };

            //Encontra o tamanho de todos os elementos
            nome.Select(f => f.Length);  // 3 3 4

            //imprimei elemento com indice
            nome.Select((a, b) => b.ToString() + ":" + a);  // 0 : um 1: dois 2:cinco

            //Imprime o item no de vezes pelo seu indice
            nome.SelectMany((a, b) => Enumerable.Repeat(a, b)).ToArray(); //dois 
        }

        public static void Particionamento()
        {
            string[] nome = { "Macoratti", ".Net", "Mac", "Visual", "Basic" };

            // pega dois elementos da lista
            List<string> valor1 = nome.Take(2).ToList<string>();         // Macoratti .Net

            //Pula o primeiro elemento na lista
            List<string> valor2 = nome.Skip(1).ToList<string>();         // .Net, Mac Visual Basic

            // Pega elementos com tamanho igual a 3
            IEnumerable<string> valor3 = nome.TakeWhile(f => f.Length == 3);  //Mac

            // Pula o elemento quando o tamanho for menor que 4
            IEnumerable<string> valor4 = nome.SkipWhile(f => f.Length < 5) ;   // .Net Mac
        }

        public static void Igualdade()
        {
             //Verificando igualdade
            string[] palavras = { "macoratti", "Visual", "Basic" };

            //Verifica se todos os itens são apresentado em sequencia
            palavras.SequenceEqual(new[] { "macoratti", "Visual", "Basic" });  //True
           
            // Caixa Alta é diferente
            palavras.SequenceEqual(new[] { "MACORATTI", "VISUAL", "BASIC" }); //False
           
            // Ignorando a caixa alta e baixa na verificação
            palavras.SequenceEqual(new[] { "MACORATTI", "VISUAL", "BASIC"}, StringComparer.OrdinalIgnoreCase);  //True
        }

        public static void OperacaoElemento()
        {
            int[] dados = { 1, 2, 3, 4, 5 };
            Console.WriteLine(dados.ElementAt(3));               //Erro
            Console.WriteLine(dados.ElementAtOrDefault(10));     //0
            Console.WriteLine(dados.First());                    //1

            string[] nome = { "Macoratti", "Mac", "Visual", "Basic" };
            //First three letter name
            Console.WriteLine(nome.First(m => m.Length == 3));   //Mac

            //Primeiro nome que é maior que 3
            Console.WriteLine(nome.First(m => m.Length > 3));   //Macoratti

            //Ultimo elemento do array
            Console.WriteLine(nome.Last());                     //Basic

            //tenta obter o item cujo tamanho é 6
            Console.WriteLine(nome.SingleOrDefault(m => m.Length == 6));  //Visual
        }

        public static void Conversao()
        {
            object[] dados = { "1", "2", "3", "4", "5" };

            //Convereter para Ienumerable<string>
            IEnumerable<string> strEnum = dados.Cast<string>();
            Console.WriteLine(strEnum.GetType());
            //Converter para List
            List<string> strList = dados.Cast<string>().ToList();
            Console.WriteLine(strList.GetType());
            //Converter para Array
            string[] strArray = dados.Cast<string>().ToArray();
            Console.WriteLine(strArray.GetType());
        }

        public static void Concatenacao()
        {
            //Primeiro array
            int[] dados = { 1, 2, 3 };

            //Concatenando com outro array
            int[] resultado = dados.Concat(new int[] { 4, 5, 6 }).ToArray(); //resultado 1,2,3,4,5,6

            foreach (var n in resultado)
            {
                Console.Write( n );
            }
        }

        public static void Agregacao()
        {
            //Examplo de agregação

            int[] dados = { 1, 2, 3, 4, 5, 6 , 7, 8 };
            //Soma de números
            Console.WriteLine(dados.Sum());     //resultado => 35
            //Média aritmética de números
            Console.WriteLine(dados.Average()); //Resultado => 4,5
            //Número Máximo
            Console.WriteLine(dados.Max());     //Resultado => 8

            String[] strdados = { "1", "dois", "tres", "quatro", "cinco" };

            //Item com o comprimento minimo
            Console.WriteLine(strdados.Min());  //Ans :1

            //Item com o tamanho minimo
            Console.WriteLine(strdados.Min(f => f.Length));  //Resultado => 1

            //Item com o tamanho maximo
            Console.WriteLine(strdados.Max(f => f.Length));  //Resultado => 6
           
        }
    }
}
